@@ -45,6 +45,7 @@
 #include <QDirIterator>
 #include <QFormLayout>
 #include <QLabel>
+#include <QRegExp>
 #include <QVBoxLayout>
 
 namespace {
@@ -63,14 +64,16 @@ static const char ANDROID_NDK_TC_VERION[] = "Qt4ProjectManager.Android.NDK_TC_VE
 QHash<Abi, QList<int> > AndroidToolChainFactory::m_newestVersionForAbi;
 FileName AndroidToolChainFactory::m_ndkLocation;
 
-AndroidToolChain::AndroidToolChain(const Abi &abi, const QString &ndkToolChainVersion, Detection d)
+AndroidToolChain::AndroidToolChain(const Abi &abi, const QString &ndkToolChainVersion, Core::Id l, Detection d)
     : GccToolChain(Constants::ANDROID_TOOLCHAIN_ID, d),
       m_ndkToolChainVersion(ndkToolChainVersion), m_secondaryToolChain(false)
 {
+    setLanguage(l);
     setTargetAbi(abi);
-    setDisplayName(QString::fromLatin1("Android GCC (%1-%2)")
-                   .arg(AndroidConfig::displayName(targetAbi()))
-                   .arg(ndkToolChainVersion));
+    setDisplayName(QString::fromLatin1("Android GCC (%1, %2-%3)")
+                   .arg(ToolChainManager::displayNameOfLanguageId(l),
+                        AndroidConfig::displayName(targetAbi()),
+                        ndkToolChainVersion));
 }
 
 // for fromMap
@@ -262,6 +265,11 @@ AndroidToolChainFactory::AndroidToolChainFactory()
     setDisplayName(tr("Android GCC"));
 }
 
+QSet<Core::Id> Android::Internal::AndroidToolChainFactory::supportedLanguages() const
+{
+    return {ProjectExplorer::Constants::CXX_LANGUAGE_ID};
+}
+
 QList<ToolChain *> AndroidToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown)
 {
     return autodetectToolChainsForNdk(AndroidConfigurations::currentConfig().ndkLocation(), alreadyKnown);
@@ -290,22 +298,24 @@ QList<AndroidToolChainFactory::AndroidToolChainInformation> AndroidToolChainFact
     QRegExp versionRegExp(NDKGccVersionRegExp);
     FileName path = ndkPath;
     QDirIterator it(path.appendPath(QLatin1String("toolchains")).toString(),
-                    QStringList() << QLatin1String("*"), QDir::Dirs);
+                    QStringList("*"), QDir::Dirs);
     while (it.hasNext()) {
         const QString &fileName = FileName::fromString(it.next()).fileName();
         int idx = versionRegExp.indexIn(fileName);
         if (idx == -1)
             continue;
-        AndroidToolChainInformation ati;
-        ati.version = fileName.mid(idx + 1);
-        QString platform = fileName.left(idx);
-        ati.abi = AndroidConfig::abiForToolChainPrefix(platform);
-        if (ati.abi.architecture() == Abi::UnknownArchitecture) // e.g. mipsel which is not yet supported
-            continue;
-        // AndroidToolChain *tc = new AndroidToolChain(arch, version, true);
-        ati.compilerCommand = AndroidConfigurations::currentConfig().gccPath(ati.abi, ati.version);
-        // tc->setCompilerCommand(compilerPath);
-        result.append(ati);
+        for (const Core::Id lang : {ProjectExplorer::Constants::CXX_LANGUAGE_ID,
+                                    ProjectExplorer::Constants::C_LANGUAGE_ID}) {
+            AndroidToolChainInformation ati;
+            ati.language = lang;
+            ati.version = fileName.mid(idx + 1);
+            QString platform = fileName.left(idx);
+            ati.abi = AndroidConfig::abiForToolChainPrefix(platform);
+            if (ati.abi.architecture() == Abi::UnknownArchitecture) // e.g. mipsel which is not yet supported
+                continue;
+            ati.compilerCommand = AndroidConfigurations::currentConfig().gccPath(ati.abi, lang, ati.version);
+            result.append(ati);
+        }
     }
     return result;
 }
@@ -347,19 +357,22 @@ bool AndroidToolChainFactory::versionCompareLess(const QList<int> &a, const QLis
     return false;
 }
 
-bool AndroidToolChainFactory::versionCompareLess(AndroidToolChain *atc, AndroidToolChain *btc)
+bool AndroidToolChainFactory::versionCompareLess(QList<AndroidToolChain *> atc,
+                                                 QList<AndroidToolChain *> btc)
 {
-    QList<int> a = versionNumberFromString(atc->ndkToolChainVersion());
-    QList<int> b = versionNumberFromString(btc->ndkToolChainVersion());
+    const QList<int> a = versionNumberFromString(atc.at(0)->ndkToolChainVersion());
+    const QList<int> b = versionNumberFromString(btc.at(0)->ndkToolChainVersion());
 
     return versionCompareLess(a, b);
 }
 
-static AndroidToolChain *findToolChain(Utils::FileName &compilerPath, const QList<ToolChain *> &alreadyKnown)
+static AndroidToolChain *findToolChain(Utils::FileName &compilerPath, Core::Id lang,
+                                       const QList<ToolChain *> &alreadyKnown)
 {
     return static_cast<AndroidToolChain *>(
-                Utils::findOrDefault(alreadyKnown, [compilerPath](ToolChain *tc) {
+                Utils::findOrDefault(alreadyKnown, [compilerPath, lang](ToolChain *tc) {
                                                        return tc->typeId() == Constants::ANDROID_TOOLCHAIN_ID
+                                                           && tc->language() == lang
                                                            && tc->compilerCommand() == compilerPath;
                                                    }));
 }
@@ -375,8 +388,8 @@ AndroidToolChainFactory::autodetectToolChainsForNdk(const FileName &ndkPath,
     QRegExp versionRegExp(NDKGccVersionRegExp);
     FileName path = ndkPath;
     QDirIterator it(path.appendPath(QLatin1String("toolchains")).toString(),
-                    QStringList() << QLatin1String("*"), QDir::Dirs);
-    QHash<Abi, AndroidToolChain *> newestToolChainForArch;
+                    QStringList("*"), QDir::Dirs);
+    QHash<Abi, QList<AndroidToolChain *>> newestToolChainForArch;
 
     while (it.hasNext()) {
         const QString &fileName = FileName::fromString(it.next()).fileName();
@@ -388,25 +401,30 @@ AndroidToolChainFactory::autodetectToolChainsForNdk(const FileName &ndkPath,
         Abi abi = AndroidConfig::abiForToolChainPrefix(platform);
         if (abi.architecture() == Abi::UnknownArchitecture) // e.g. mipsel which is not yet supported
             continue;
-        FileName compilerPath = AndroidConfigurations::currentConfig().gccPath(abi, version);
+        QList<AndroidToolChain *> toolChainBundle;
+        for (Core::Id lang : {ProjectExplorer::Constants::CXX_LANGUAGE_ID, ProjectExplorer::Constants::C_LANGUAGE_ID}) {
+            FileName compilerPath = AndroidConfigurations::currentConfig().gccPath(abi, lang, version);
 
-        AndroidToolChain *tc = findToolChain(compilerPath, alreadyKnown);
-        if (!tc) {
-            tc = new AndroidToolChain(abi, version, ToolChain::AutoDetection);
-            tc->resetToolChain(compilerPath);
+            AndroidToolChain *tc = findToolChain(compilerPath, lang, alreadyKnown);
+            if (!tc) {
+                tc = new AndroidToolChain(abi, version, lang,
+                                          ToolChain::AutoDetection);
+                tc->resetToolChain(compilerPath);
+            }
+            result.append(tc);
+            toolChainBundle.append(tc);
         }
-        result.append(tc);
 
         auto it = newestToolChainForArch.constFind(abi);
         if (it == newestToolChainForArch.constEnd())
-            newestToolChainForArch.insert(abi, tc);
-        else if (versionCompareLess(it.value(), tc))
-            newestToolChainForArch[abi] = tc;
+            newestToolChainForArch.insert(abi, toolChainBundle);
+        else if (versionCompareLess(it.value(), toolChainBundle))
+            newestToolChainForArch[abi] = toolChainBundle;
     }
 
     foreach (ToolChain *tc, result) {
         AndroidToolChain *atc = static_cast<AndroidToolChain *>(tc);
-        atc->setSecondaryToolChain(newestToolChainForArch.value(atc->targetAbi()) != atc);
+        atc->setSecondaryToolChain(!newestToolChainForArch.value(atc->targetAbi()).contains(atc));
     }
 
     return result;
@@ -420,7 +438,7 @@ QList<int> AndroidToolChainFactory::newestToolChainVersionForArch(const Abi &abi
         m_ndkLocation = AndroidConfigurations::currentConfig().ndkLocation();
         FileName path = m_ndkLocation;
         QDirIterator it(path.appendPath(QLatin1String("toolchains")).toString(),
-                        QStringList() << QLatin1String("*"), QDir::Dirs);
+                        QStringList("*"), QDir::Dirs);
         while (it.hasNext()) {
             const QString &fileName = FileName::fromString(it.next()).fileName();
             int idx = versionRegExp.indexIn(fileName);
